//==================================================
// AntMotor.cpp 
// Arduino library to control the variAnt Robot-Ant
// Copyright (c) 2022 Tino Werner / VARIOBOT
//==================================================

#include "AntMotor.h"

AntMotor::AntMotor(byte pinA, byte pinB, byte pinR) 
{
  this->pinA = pinA;
  this->pinB = pinB;
  this->pinR = pinR;

  pinMode(pinA, OUTPUT);
  pinMode(pinB, OUTPUT);
  pinMode(pinR, INPUT);  
}

void AntMotor::setup(byte acceleration, byte syncSlowDown)
{
  this->acceleration = constrain(acceleration, 1, 10);
  this->syncSlowDown = constrain(syncSlowDown, 1, 10); // a value of 1 switches of synchronization
}

//----------------------------------

bool AntMotor::nextStep()
{
  if(reed == (reed=digitalRead(pinR))) // reed hasn't switched
    return false; // no step done

  if(!reed) // reed has switched off
    reedMillis = millis(); // save the time when reed turned off
  else // reed has switched on
    if(millis()-reedMillis < 30000/abs(speed)) // maximum duration for TOP position
      position = TOP; // set top position

  const pos priorPosition = position;       // save the last leg position
  position = (position+1+2*(speed<0)) % 4;  // set next (forward or backward) position
  
  return bool(position) != bool(priorPosition); // returns true if a step has done
}

void AntMotor::syncSteps(AntMotor* otherMotor) 
{   
  if(nextStep()) // if a step has done
  {
    if(otherMotor->slow) 
      otherMotor->slow = false; // speed up the other motor again
    else 
      if(bool(this->position) == bool(otherMotor->position)) // this motor is ahead
        this->slow = true; // slow down the motor speed
 
    if(speed < 0 && stepsBack > 0 && --stepsBack == 0) // last backward step done
      setDirection(FORWARD);
  }
}

void AntMotor::updateSpeed() 
{       
  const int slowerSpeed = targetSpeed / (slow ? syncSlowDown : 1); 
  const int speedChange = acceleration * (millis() - lastMillis); 

  lastMillis = millis(); 
  
  if(speed < slowerSpeed) 
  {
    speed = min(speed+speedChange, slowerSpeed);
    setPWM(); // update the increased motor speed
  }
  if(speed > slowerSpeed) 
  {
    speed = max(speed-speedChange, slowerSpeed);
    setPWM(); // update the decreased motor speed
  }
}

//--------------------------------------------

byte AntMotor::getSpeed()
{
  return abs(targetSpeed);
}

void AntMotor::setSpeed(byte newSpeed)
{
  if(targetSpeed >= 0)
    targetSpeed = newSpeed;
  else
    targetSpeed = -newSpeed;
}

void AntMotor::setDirection(bool newDir)
{
  if(newDir == BACKWARD ^ targetSpeed < 0) // newDir and direction of targetSpeed are different
  {
    targetSpeed *= -1; // switches the direction of targetSpeed
    slow = false;
  }
}

void AntMotor::stepBack(byte steps)
{
  if(steps > 0) 
  {
    stepsBack = steps;  
    setDirection(BACKWARD);
  }
}

void AntMotor::slowDown()
{
  if(stepsBack == 0) // if not stepping backward
    slow = true;     // slow down the motor speed
}

//-------------------------------

void AntMotor::setPWM()
{
  if(speed >= 0) 
    digitalWrite(pinA, HIGH);

  if(speed <= 0) 
    digitalWrite(pinB, HIGH);
  
  if(speed > 0) 
    analogWrite(pinB, 255-speed);
  
  if(speed < 0) 
    analogWrite(pinA, 255+speed);
}
