//==================================================
// VariAnt.cpp 
// Arduino library to control the variAnt Robot-Ant
// Copyright (c) 2022 Tino Werner / VARIOBOT
//==================================================

#include "VariAnt.h"

//--------------------------------------------------------------------
// The constructor function of the VariAnt class 
//--------------------------------------------------------------------

VariAnt::VariAnt( void ) : 
  motor {{motorPinA[0], motorPinB[0], reedPin[0]},
         {motorPinA[1], motorPinB[1], reedPin[1]}}
{ 
}

//-----------------------------------------------------------------------------
// The setup() function must be called once at the beginning 
// acceleration: speed change per millisecond 
// syncSlowDown: speed reduction for synchronizing the motors 
//-----------------------------------------------------------------------------

void VariAnt::setup(byte acceleration, byte syncSlowDown)
{ 
  //set timer 1 divisor for D9 & D10 to 1 for PWM frequency of 31372.55 Hz
  TCCR1B = TCCR1B & B11111000 | B00000001; 
  //set timer 2 divisor for D3 & D11 to 1 for PWM frequency of 31372.55 Hz  
  TCCR2B = TCCR2B & B11111000 | B00000001; 

  //set the (Arduino standard) resolution for the analog pins 
  #if defined(__LGT8F__) // if using an LGT8F Board
    analogReadResolution(10); // 2^10 = 1024
  #endif

  //initialize the analog pins as input
  for(int i = 0; i <= 7; i++)
    pinMode(analogPin[i], INPUT);

  //initialize the digital pins as output
  for(int i = L3; i <= R3; i++)
    pinMode(digitalPin[i], OUTPUT);

  //set acceleration and syncSlowDown for the motors
  motor[0].setup(acceleration, syncSlowDown);
  motor[1].setup(acceleration, syncSlowDown);
}

//-------------------------------------------------------------------------------
// The update() function must be called regularly inside the loop function
// It synchronizes the speed of the motors, counts the running steps to be done,
// calculates the sensor values, checks the buttons, and updates the LED outputs.
//-------------------------------------------------------------------------------

void VariAnt::update()
{
  int i;
  //read the analog inputs
  for(i=0; i<=7; i++)  
    input[i] = analogRead(analogPin[i]);

  //calcualte the sensor values
  sensor[0] = input[0];
  for(i=1; i<=7; i++)  
    sensor[i] = input[i] - input[i-1];
  sensor[8] = 1023 - input[7];

  //update the analog or digital LED outputs
  for(i=L3; i<=R3; i++)
    if((i==L0 || i==R0) && !led[i])
      analogWrite(digitalPin[i], led0[i>M]); 
    else if(i!=M)
    { 
      digitalWrite(digitalPin[i], led[i]); 
      led[i] = LOW; // set default LED outputs to LOW
    }

  for(byte s=0; s<=1; s++)
  { 
    //synchronizing with the other motor and steps backward
    motor[s].syncSteps(&motor[!s]);
    //update the fading speed of the motor
    motor[s].updateSpeed();

    // if the tactile switch is pressed the corrensponding inputs are equal
    if(!button[s] && input[5*s+2] == input[5*s]) // button just pressed
    {
      button[s] = true;
      buttonMillis[s] = millis();    
    }
    if(button[s] && input[5*s+2]-input[5*s] > 9) // just released
    {
      button[s] = false;
      buttonReleased[s] = true;
    }
    else
      buttonReleased[s] = false; 
  }
}

//---------------------------------------------------------------------
// functions to change the movement and speed of variAnt
//---------------------------------------------------------------------

byte VariAnt::getSpeed(bool side)
{
  return motor[side].getSpeed();
}
 
void VariAnt::setSpeed(byte newSpeed)
{
  motor[0].setSpeed(newSpeed);
  motor[1].setSpeed(newSpeed);
}
 
void VariAnt::setSpeed(byte leftSpeed, byte rightSpeed)
{
  motor[0].setSpeed(leftSpeed);
  motor[1].setSpeed(rightSpeed);
}

void VariAnt::changeSpeed(int speedChange)
{
  motor[0].setSpeed(constrain(motor[0].getSpeed()+speedChange, 0, 255));
  motor[1].setSpeed(constrain(motor[1].getSpeed()+speedChange, 0, 255));
}

void VariAnt::run(bool dir)
{ 
  runningDir = dir;  
  motor[0].setDirection(dir);
  motor[1].setDirection(dir);
}

void VariAnt::turn(bool s)
{ 
  turningDir = s;  
  motor[0].setDirection(!s);
  motor[1].setDirection(s);
}

void VariAnt::stepBack(byte leftSteps, byte rightSteps) 
{
  motor[0].stepBack(leftSteps);
  motor[1].stepBack(rightSteps);
}

void VariAnt::slowDown(bool s)
{ 
  motor[s].slowDown();
}

//---------------------------------------------------------------------
// sensor functions 
//---------------------------------------------------------------------

int VariAnt::getSensor(pos i)
{
  return sensor[i];
}

void VariAnt::printSensors(int interval)
{
  if(millis() - printMillis >= interval)
  {
    printMillis = millis(); 
    Serial.println(); 
    Serial.println("  Sensors");
    Serial.println("left   right"); 
    Serial.println("  "+(String)sensor[L0]+"   "+(String)sensor[R0]); 
    Serial.println("   "+(String)sensor[L2]+" "+(String)sensor[R2]); 
    Serial.println((String)sensor[L1]+"       "+(String)sensor[R1]); 
    Serial.println((String)sensor[L3]+"       "+(String)sensor[R3]); 
    Serial.println("    "+(String)sensor[M]);
    Serial.println(); 
  }
}

//--------------------------------------------------------------------------------
// senseShadow() compares the sum of the selected sensor values with a threshold
// if the threshold is reached, the associated LEDs are activated 
//--------------------------------------------------------------------------------

bool VariAnt::senseShadow(int threshold, pos s1, pos s2, pos s3, pos s4)
{
  //if a button is pressed the senseShadow function is blocked
  if (button[0] || button[1])
    return false;

  int value = sensor[s1];
  if(s2 <= R3)
    value += sensor[s2];
  if(s3 <= R3)
    value += sensor[s3];
  if(s4 <= R3)
    value += sensor[s4];

  //activate the LEDs associated with the selected sensors
  if(value >= threshold)
  {
    led[s1] = HIGH;
    if(s2 <= R3)
      led[s2] = HIGH;
    if(s3 <= R3)
      led[s3] = HIGH;
    if(s4 <= R3)
      led[s4] = HIGH;

    return true;
  }
  return false;
}

//---------------------------------------------------------------------
// button functions 
//---------------------------------------------------------------------

bool VariAnt::getButton(bool s)
{
  return button[s];
}

bool VariAnt::buttonPressed(bool s)
{
  return buttonReleased[s];
}

bool VariAnt::buttonPressed(bool s, bool longPressed)
{
  return buttonReleased[s] && longPressed == (millis()-buttonMillis[s] > BUTTON_MILLIS);
}

//---------------------------------------------------------------------
// LED functions 
//---------------------------------------------------------------------

void VariAnt::setLed(pos i, bool value)
{
  led[i] = value; 
}

void VariAnt::setLeds(byte valueL, byte valueR)
{
  led0[LEFT]  = valueL; 
  led0[RIGHT] = valueR; 
}

void VariAnt::showShadows(int threshold)
{
  for(int i=L3; i<=R3; i++)
    if(sensor[i] > threshold)
      led[i] = HIGH; 
}

void VariAnt::showSensors(pos s1, pos s2)
{
  led0[0] = sensor[s1] >>2; 
  led0[1] = sensor[s2] >>2; 
}
