//==================================================
// VariAnt.h 
// Arduino library to control the variAnt Robot-Ant
// Copyright (c) 2022 Tino Werner / VARIOBOT
//==================================================

#ifndef VariAnt_h
#define VariAnt_h

#include <Arduino.h>
#include "AntMotor.h"

const bool FORWARD        = 0;                          // forward direction for motor and for variAnt
const bool BACKWARD       = 1;                          // reverse direction for motor and for variAnt
const bool LEFT           = 0;                          // left side of robot or left direction of turning
const bool RIGHT          = 1;                          // right side of robot or left direction of turning
const bool SHORT          = 0;                          // short duration of button press (< 500 ms)
const bool LONG           = 1;                          // long duration of button press (>= 500 ms)
const int  BUTTON_MILLIS  = 500;                        // defines max/min millis for SHORT/LONG button push
enum  pos {L3,L2,L1,L0,M,R0,R1,R2,R3, NO};              // identifiers for the sensor positions and the coresponding LEDs

//--------------------------------------------------

class VariAnt
{
public:

  VariAnt();                                            // Constructor

  void setup(byte acceleration=1, byte syncSlowDown=2); // Initialisation must be called at the beginnig
  void update();                                        // must be called regularly within the loop function

  byte getSpeed(bool side=RIGHT);                       // returns the absolute target speed of the left or right motor
  void setSpeed(byte newSpeed);                         // same speed of both motors (always >= 0)
  void setSpeed(byte leftSpeed, byte rightSpeed);       // set different speed for the left and right motor
  void changeSpeed(int speedChange);                    // the speedChange can be < 0 or > 0
  
  void run(bool dir);                                   // let variAnt run FORWARD or BACKWARD
  void turn(bool s);                                    // let variAnt turn LEFT or RIGHT
  void stepBack(byte stepsLeft, byte stepsRight);       // steps back a specific number of steps on each side
  void slowDown(bool s);                                // slows down the speed of one motor until the next step has done

  int  getSensor(pos i);                                // get the value of sensor[i]
  void printSensors(int interval);                      // print the values of the 9 sensors at the interval
  bool senseShadow(int th, pos s1, pos s2=NO, pos s3=NO, pos s4=NO);// true, if the sum of the sensor values reaches a threshold

  bool getButton(bool s);                               // checks whether the button is currently pressed or not
  bool buttonPressed(bool s);                           // checks whether the button was just pressed
  bool buttonPressed(bool s, bool longPressed);         // checks whether the button was just pressed for short or long time (BUTTON_MILLIS)

  void setLed(pos i, bool value=HIGH);                  // set digital Led value, by default: led[i] = HIGH
  void setLeds(byte valueL, byte valueR);               // set analog PWM values for led0[LEFT] and led0[RIGHT]
  void showShadows(int threshold);                      // corresponding LEDs will be on if the sensor values exceed the threshold
  void showSensors(pos s1=L0, pos s2=R0);               // PWM-LEDs showing values of 2 sensors - by default: sensor[L0] and sensor[R0]

private:

  const byte reedPin[2]    = {8,2};                     // pins for the left and right reed switches
  const byte motorPinA[2]  = {9,3};                     // Ahead pins for the left and right motor
  const byte motorPinB[2]  = {10,11};                   // Back pins for the left and right motor
  const byte analogPin[8]  = {14,15,16,17,18,19,20,21}; // analog input pins for the left and right sensors
  const byte digitalPin[9] = {13,7,12,6,0,5,0,4,1};     // digital output pins for the left and right LEDs

  bool runningDir;                                      // direction of running can be FORWARD(0) or BACKWARD(1)
  bool turningDir;                                      // direction of turning can be LEFT(0) or RIGHT(1)

  AntMotor motor[2];                                    // motor objects for the left and right motor

  int input[8];                                         // analog input values                              
  int sensor[9];                                        // values of the 9 sensors
  
  bool button[2];                                       // status of the buttons (0: not pressed, 1: pressed)
  bool buttonReleased[2];                               // status of the falling edge (1: just released)

  bool led[9];                                          // digital values of the 8 LED outputs
  byte led0[2];                                         // PWM values of the first left and right LED output

  long printMillis;                                     // saves the current millis when the sensor status is printed
  long buttonMillis[2];                                 // saves the current millis when the button is pressed
};

#endif